import argparse
import glob
import os
import random
import numpy as np
import torch
import torch.nn.functional as F
import torchvision.transforms.functional as T
from PIL import Image
from torch.backends import cudnn
from torch.utils.data import Dataset
from torchvision.transforms import RandomCrop
import torchvision
import torch.nn as nn
from torch import nn as nn
from torch.nn import functional as F


def parse_args():
    desc = 'Pytorch Implementation of \'Frequency Modulated Deformable Transformer for Underwater Image Enhancement\''
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument('--data_path', type=str, default='./')
    parser.add_argument('--data_path_test', type=str, default='./datasets/images/')
    parser.add_argument('--data_name', type=str, default='UIEB', choices=['sea-thruc','C_60','color_checker','UIEB','EUVP'])
    parser.add_argument('--save_path', type=str, default='result')
    parser.add_argument('--inp_channels', type=int, default=3,
                        help='number of input channels')
    parser.add_argument('--out_channels', type=int, default=3,
                        help='number of output channels')
    parser.add_argument('--dim', type=int, default=8,
                        help='number of channels ')
    parser.add_argument('--num_blocks', nargs='+',type=float, default=[1, 1, 2], help='number of transformer blocks for each level')
  
    parser.add_argument('--num_refinement_blocks', type=int, default=4, help='number of channels for refinement stage')
    parser.add_argument('--ffn_expansion_factor', type=int, default=3, help='')
    parser.add_argument('--bias', default=False,
                        help='')
    parser.add_argument('--patch_size', nargs='+', type=int, default=[256, 256, 256, 256],
                        help='patch size of each image')
    parser.add_argument('--batch_size', nargs='+', type=int, default=[1,1,1,1],
                        help='batch size of loading images ')
    parser.add_argument('--lr', type=float, default=0.0003, help='initial learning rate')
    parser.add_argument('--milestone', type=int, default=[270000], help='when to change patch size and batch size')
    parser.add_argument('--workers', type=int, default=0, help='number of data loading workers')
    parser.add_argument('--seed', type=int, default=-1, help='random seed (-1 for no manual seed)')
        #model_file is None means training stage, else means testing stage
    parser.add_argument('--model_file', type=str, default='./checkpoint/UIEB.pth', help='path of pre-trained model file')
    parser.add_argument('--finetune', default=True, help='path of pre-trained model file')
    parser.add_argument('--num_iter',type=int, default=171000, help='iteration of training')
        




    return init_args(parser.parse_args())


class Config(object):
    def __init__(self, args):
        self.data_path = args.data_path
        self.data_path_test = args.data_path_test
        self.data_name = args.data_name
        self.save_path = args.save_path
        self.inp_channels = args.inp_channels
        self.out_channels = args.out_channels
        self.dim = args.dim
        self.num_blocks = args.num_blocks
        self.num_refinement_blocks = args.num_refinement_blocks
        self.ffn_expansion_factor = args.ffn_expansion_factor
        self.bias = args.bias
        self.patch_size = args.patch_size
        self.batch_size=args.batch_size
        self.lr = args.lr
        self.milestone = args.milestone
        self.workers = args.workers
        self.model_file = args.model_file
        self.finetune = args.finetune
        self.num_iter=args.num_iter
       
        


def init_args(args):
    if not os.path.exists(args.save_path):
        os.makedirs(args.save_path)

    if args.seed >= 0:
        random.seed(args.seed)
        np.random.seed(args.seed)
        torch.manual_seed(args.seed)
        torch.cuda.manual_seed_all(args.seed)
        cudnn.deterministic = True
        cudnn.benchmark = False

    return Config(args)


def pad_image_needed(img, size):
    width, height = T.get_image_size(img)
    if width < size[1]:
        img = T.pad(img, [size[1] - width, 0], padding_mode='reflect')
    if height < size[0]:
        img = T.pad(img, [0, size[0] - height], padding_mode='reflect')
    return img


class UIEDataset(Dataset):
    def __init__(self, data_path, data_path_test, data_name, data_type, patch_size=None, length=None):
        super().__init__()
        self.data_name, self.data_type, self.patch_size = data_name, data_type, patch_size
        self.UIE_images = sorted(glob.glob('{}/*.png'.format(data_path)))
        self.UIE_images_test = sorted(glob.glob('{}/*.png'.format(data_path_test)))
        self.num = len(self.UIE_images)
        self.num_test = len(self.UIE_images_test)
        self.sample_num = length if data_type == 'train' else self.num

    def __len__(self):
        return self.sample_num

    def __getitem__(self, idx):

        
        image_name = os.path.basename(self.UIE_images_test[idx % self.num_test])

        imag = np.array(Image.open(self.UIE_images_test[idx % self.num_test]).resize((1024,512)))

        r, c, ch = imag.shape
        width = c // 2

        Input = T.to_tensor(imag[:, :width, :])
        gt = T.to_tensor(imag[:, width:width * 2, :])

    
        h, w = Input.shape[1:]

           
        return Input, gt, image_name, h, w





